from typing import List

from nisystemlink.clients.core._api_error import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._create_test_plan_template_request import CreateTestPlanTemplateRequest
from ._test_plan_templates import TestPlanTemplate


class CreateTestPlanTemplatePartialSuccessResponse(JsonModel):

    created_test_plan_templates: List[TestPlanTemplate] | None = None
    """The list of test plan templates that were successfully created."""

    failed_test_plan_templates: List[CreateTestPlanTemplateRequest] | None = None
    """The list of test plan templates that were not created.

    If this is `None`, then all test plan templates were successfully created.
    """

    error: ApiError | None = None
    """Error messages for test plan templates that were not created.

    If this is `None`, then all test plan templates were successfully created.
    """



from typing import List

from nisystemlink.clients.testmonitor.models._create_steps_request import (
    BaseStepRequest,
)


class UpdateStepRequest(BaseStepRequest):
    name: str | None = None
    """Step name."""

    children: List["UpdateStepRequest"] | None = None
    """Nested child steps."""



from datetime import datetime
from typing import Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel
from nisystemlink.clients.testmonitor.models._status import Status


class UpdateResultRequest(JsonModel):
    """Contains information about a result."""

    status: Status | None = None
    """The status of the result."""

    started_at: datetime | None = None
    """The time that the result started."""

    program_name: str | None = None
    """The name of the program that generated this result."""

    id: str
    """The globally unique id of the result."""

    system_id: str | None = None
    """The id of the system that generated this result."""

    host_name: str | None = None
    """The name of the host that generated this result."""

    part_number: str | None = None
    """The part number is the unique identifier of a product within a single org."""

    serial_number: str | None = None
    """The serial number of the system that generated this result."""

    total_time_in_seconds: float | None = None
    """The total time that the result took to run in seconds."""

    keywords: List[str] | None = None
    """A list of keywords that categorize this result."""

    properties: Dict[str, str | None] | None = None
    """A list of custom properties for this result."""

    operator: str | None = None
    """The operator that ran the result."""

    file_ids: List[str] | None = None
    """A list of file ids that are attached to this result."""

    data_table_ids: List[str] | None = None
    """A list of data table ids that are attached to this result."""

    workspace: str | None = None
    """The id of the workspace that this product belongs to."""



from typing import Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel
from pydantic import Field

from ._asset import (
    AssetBusType,
    AssetDiscoveryType,
    AssetType,
    ExternalCalibration,
    SelfCalibration,
    TemperatureSensor,
)
from ._asset_location import AssetLocationForCreate


class CreateAssetRequest(JsonModel):
    """Model for an object describing the properties for creating an asset."""

    model_name: str | None = None
    """Gets or sets model name of the asset."""

    model_number: int | None = None
    """Gets or sets model number of the asset."""

    serial_number: str | None = None
    """Gets or sets serial number of the asset."""

    vendor_name: str | None = None
    """Gets or sets vendor name of the asset."""

    vendor_number: int | None = None
    """Gets or sets vendor number of the asset."""

    bus_type: AssetBusType | None = None
    """Gets or sets all supported bus types for an asset."""

    name: str | None = None
    """Gets or sets name of the asset."""

    asset_type: AssetType | None = None
    """Gets or sets all supported asset types."""

    firmware_version: str | None = None
    """Gets or sets firmware version of the asset."""

    hardware_version: str | None = None
    """Gets or sets hardware version of the asset."""

    visa_resource_name: str | None = None
    """Gets or sets VISA resource name of the asset."""

    temperature_sensors: List[TemperatureSensor] | None = None
    """Gets or sets an array of temperature sensor information."""

    supports_self_calibration: bool | None = None
    """Gets or sets whether the asset supports self-calibration."""

    supports_external_calibration: bool | None = None
    """Gets or sets whether the asset supports external calibration."""

    custom_calibration_interval: int | None = None
    """Gets or sets the interval represented in months used for computing calibration due date."""

    self_calibration: SelfCalibration | None = None
    """Gets or sets the last self-calibration of the asset."""

    is_NI_asset: bool | None = Field(alias="isNIAsset", default=None)
    """Gets or sets whether this asset is an NI asset (true) or a third-party asset (false)."""

    workspace: str | None = None
    """Gets or sets the ID of the workspace."""

    location: AssetLocationForCreate
    """Model for information about the asset location, presence and the connection status of the system"""

    external_calibration: ExternalCalibration | None = None
    """Gets or sets the last external calibration of the asset."""

    properties: Dict[str, str] | None = None
    """	Gets or sets key-value-pair metadata associated with an asset."""

    keywords: List[str] | None = None
    """Gets or sets words or phrases associated with an asset."""

    discovery_type: AssetDiscoveryType | None = None
    """Gets or sets the discovery type."""

    file_ids: List[str] | None = None
    """Gets or sets all files linked to the asset."""

    supports_self_test: bool | None = None
    """Gets or sets whether the asset supports self-test."""

    supports_reset: bool | None = None
    """Gets or sets whether the asset supports reset."""

    part_number: str | None = None
    """Gets or sets part number of the asset."""



from enum import Enum


class FileQueryOrderBy(Enum):
    """Order Files Query Response by Metadata for GET /files endpoint."""

    CREATED = "created"
    ID = "id"
    SIZE = "size"
    LAST_UPDATED_TIMESTAMP = "lastUpdatedTimestamp"


class FileLinqQueryOrderBy(Enum):
    """Order Files LINQ Query by Metadata for POST /query-files-linq endpoint."""

    CREATED = "created"
    UPDATED = "updated"
    EXTENSION = "extension"
    SIZE = "size"
    WORKSPACE = "workspace"



from typing import List

from nisystemlink.clients.core._api_error import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel


class DeleteTestPlanTemplatesPartialSuccessResponse(JsonModel):
    """The result of deleting multiple test plan templates
    when one or more test plan templates could not be deleted.
    """

    deleted_test_plan_template_ids: List[str]
    """The IDs of the test plan template that could not be deleted."""

    failed_test_plan_template_ids: List[str] | None = None
    """The IDs of the test plan template that could not be deleted."""

    error: ApiError | None = None
    """The error that occurred when deleting the test plan template."""



# mypy: disable-error-code = misc

from types import UnionType
from typing import Any, Callable, Dict, get_origin, Type, Union

import requests
from nisystemlink.clients import core
from pydantic import TypeAdapter
from requests import JSONDecodeError, Response
from uplink import commands, Consumer, converters, response_handler, utils
from uplink.auth import BasicAuth

from ._json_model import JsonModel


@response_handler
def _handle_http_status(response: Response) -> Response | None:
    """Checks an HTTP response's status code and raises an exception if necessary."""
    if 200 <= response.status_code < 300:
        # Return None for "204 No Content" responses.
        if response.status_code == 204:
            return None
        return response

    msg = "Server responded with <{} {}> ({}).".format(
        response.status_code, response.reason, response.url
    )

    try:
        content = response.json()
        if content and "error" in content:
            err_obj = core.ApiError.model_validate(content["error"])
        else:
            err_obj = None

        raise core.ApiException(
            msg, error=err_obj, http_status_code=response.status_code
        )
    except JSONDecodeError:
        if response.text:
            msg += ":\n\n" + response.text
        raise core.ApiException(msg, http_status_code=response.status_code)


_type_adapters: Dict[Type, TypeAdapter] = dict()


class _JsonModelConverter(converters.Factory):
    """A converter that converts between JSON and Pydantic models."""

    def __init__(self) -> None:
        super().__init__()

    def create_request_body_converter(
        self, _class: Type, _: commands.RequestDefinition
    ) -> Callable[[JsonModel], Dict] | None:
        def encoder(model: JsonModel) -> Dict:
            return model.model_dump(mode="json", by_alias=True, exclude_unset=True)

        if utils.is_subclass(_class, JsonModel):
            return encoder
        else:
            return None

    def create_response_body_converter(
        self, _class: Type, _: commands.RequestDefinition
    ) -> Callable[[Response], Any] | None:
        def decoder(response: Response | Any) -> Any:
            if response is None:
                return None

            adapter = _type_adapters[_class]
            if isinstance(response, Response):
                if response.status_code == 204:
                    return None
                return adapter.validate_json(response.text, by_alias=True, strict=True)
            else:
                # In cases where a return_key is specified, the response will already be parsed into a dict
                return adapter.validate_python(response, by_alias=True, strict=True)

        origin = get_origin(_class)
        modelable_origin = (
            origin is Union or origin is UnionType or origin is dict or origin is list
        )
        if modelable_origin or utils.is_subclass(_class, JsonModel):
            if _type_adapters.get(_class) is None:
                _type_adapters[_class] = TypeAdapter(_class)
            return decoder
        else:
            return None


class BaseClient(Consumer):
    """Base class for SystemLink clients, built on top of `Uplink <https://github.com/prkumar/uplink>`_."""

    def __init__(self, configuration: core.HttpConfiguration, base_path: str = ""):
        """Initialize an instance.

        Args:
            configuration: Defines the web server to connect to and information about how to connect.
            base_path: The base path for all API calls.
        """
        session = requests.Session()
        session.verify = configuration.verify
        auth: BasicAuth | None = None
        if (configuration.username is not None) and (
            configuration.password is not None
        ):
            auth = BasicAuth(configuration.username, configuration.password)

        super().__init__(
            base_url=configuration.server_uri + base_path,
            converter=_JsonModelConverter(),
            hooks=[_handle_http_status],
            client=session,
            auth=auth,
        )
        if configuration.api_keys:
            self.session.headers.update(configuration.api_keys)



from __future__ import annotations

from datetime import datetime
from typing import List

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._platform import Platform


class Feed(JsonModel):
    """Feed model."""

    id: str | None = None
    """The auto-generated ID of the feed."""
    name: str | None = None
    """The name of the feed."""
    description: str | None = None
    """The description of the feed."""
    platform: Platform | None = None
    """The platform of the feed, the following package extensions are available: .nipkg for
    windows feeds, .ipk and .deb for ni-linux-rt feeds.
    """
    workspace: str | None = None
    """The ID of the workspace this feed belongs to."""
    updated_at: datetime | None = None
    """The date of the latest feed update"""
    created_at: datetime | None = None
    """The date when the feed was created at."""
    package_sources: List[str] | None = None
    """The package sources list of the feed."""
    deleted: bool | None = None
    """Whether the feed deletion was requested."""
